/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: java.cxx 5971 2009-03-01 17:14:58Z jz $

#include "zschimmer.h"
#include "threads.h"
#include "z_com.h"
#include "java.h"

using namespace std;

namespace zschimmer {
namespace javabridge {

//-------------------------------------------------------------------------------------------static

const raw_parameter_list<0> empty_parameterlist;
const Parameter_list& Parameter_list::empty = empty_parameterlist;

//------------------------------------------------------------Class_register::get_or_register_class
    
Class* Class_register::get_or_register_class( const Class_factory* class_factory )
{
    Class* result = get( class_factory );
    if( !result )
        result = register_new_class( class_factory );

    return result;
}

//------------------------------------------------------------------------------Class_register::get
    
Class* Class_register::get( const Class_factory* class_factory )
{
    Class* result = NULL;
    const string& class_name = class_factory->class_name();

    Z_FAST_MUTEX( _mutex ) {    // Nur Lesezugriff
        if( Entry* entry = find_map_entry( _map, class_name ) ) {
            if( entry->_class_factory != class_factory )  
                throw_xc( Z_FUNCTION, class_name );     // Dieselbe Klasse mit zwei Factory definiert?
            result = entry->_class;
        }
    }

    return result;
}

//---------------------------------------------------------------Class_register::register_new_class

Class* Class_register::register_new_class( const Class_factory* class_factory )
{
    Class* result = NULL;
    ptr<Class> clas = class_factory->new_instance();

    Z_FAST_MUTEX( _mutex ) {    // Schreibzugriff
        Entry* entry = &_map[ class_factory->class_name() ];
        entry->_class_factory = class_factory;
        entry->_class = class_factory->new_instance();

        result = entry->_class;
    }

    return result;
}

//------------------------------------------------------------------------------------Value::~Value

Value::~Value()
{ 
    set_simple_type( t_void ); 
}

//------------------------------------------------------------------------------Value::assign_value
    
void Value::assign_value( const Value& o )
{
    if( o._type == t_object )
        assign_jobject( o.as_jobject() );
    else {
        _type = o._type;
        _jvalue = o._jvalue;
    }
}

//----------------------------------------------------------------------------Value::assign_jobject

void Value::assign_jobject( jobject value )
{
    if( _type != t_object )  
        set_simple_type( t_object );

    if( _jvalue.l != value )
        set_global_ref( value ); 
}
                                                              
//-----------------------------------------------------------------------------Value::assign_string

void Value::assign_string( const char* str, int length )
{
    Env env;
    steal_local_ref( env.jstring_from_string( str, length ) );
}
                                                              
//---------------------------------------------------------------------------Value::set_simple_type

void Value::set_simple_type( Simple_type type )
{
    if( _type == t_object )
        set_global_ref( NULL );
    
    _type = type;
    _jvalue.l = NULL;
}

//-------------------------------------------------------------------------Value::check_simple_type

void Value::check_simple_type( const Simple_type& type ) const
{
    if( _type == type )
        return;

    throw_xc( "Z-JAVA-114", string( 1, (char)type ) );
}

//--------------------------------------------------------------------------------Value::check_type

void Value::check_type( const Type& type ) const
{
    if( type.simple_type() == t_object )
        if( Class::of_object( _jvalue.l )->name() == type.signature_string() )
            return;

    if( _type == type.simple_type() )
        return;

    throw_xc( "Z-JAVA-114", type.signature_string() );
}

//----------------------------------------------------------------------------Value::set_global_ref

void Value::set_global_ref( jobject jo )
{
    assert( _type == t_object );

    if( _jvalue.l != jo ) 
    {
        Env env;

        jobject new_jobject = NULL;

        if( jo )
        {
            Z_LOG2( Vm::java_log_category, "NewGlobalRef()\n" );
            new_jobject = env->NewGlobalRef( jo );
            if( !new_jobject || env->ExceptionCheck() )  env.throw_java( "NewGlobalRef" );

            //?? env->DeleteLocalRef( jo );    // jo wird ungltig!
        }

        if( _jvalue.l )
        {
            Z_LOG2( Vm::java_log_category, "DeleteGlobalRef()\n" );
            env->DeleteGlobalRef( _jvalue.l );
            _jvalue.l = NULL;
        }

        _jvalue.l = new_jobject;
    }
}

//---------------------------------------------------------------------------Value::steal_local_ref

void Value::steal_local_ref( jobject jo )
{
    assign_jobject( jo );

    if( jo ) {
        Env env;
        assert( env->GetObjectRefType( jo ) == JNILocalRefType );
        Z_LOG2( Vm::java_log_category, "DeleteLocalRef()\n" );
        env->DeleteLocalRef( jo );
    }
}

//-----------------------------------------------------------------------------Value::set_local_ref

//void Value::set_local_ref( jobject jo )
//{
//    assert( _type == t_object );
//
//    if( _jvalue.l != jo ) 
//    {
//        Env env;
//
//        jobject new_jobject = NULL;
//
//        if( jo )
//        {
//            //TODO Soll jo am Ende freigegeben werden? Besser explizite Methode takeFrom()?
//            new_jobject = jo;
//            //Z_LOG2( Vm::java_log_category, "NewLocalRef()\n" );
//
//            //new_jobject = env->NewLocalRef( jo );
//            //if( !new_jobject || env->ExceptionCheck() )  env.throw_java( "NewLocalRef" );
//
//            //env->DeleteLocalRef( jo );    // jo wird ungltig!
//        }
//
//        if( _jvalue.l )
//        {
//            Z_LOG2( Vm::java_log_category, "DeleteLocalRef()\n" );
//            env->DeleteLocalRef( _jvalue.l );
//            _jvalue.l = NULL;
//        }
//
//        _jvalue.l = new_jobject;
//    }
//}

//---------------------------------------------------------------------------------Value::as_string

string Value::as_string() const
{
    //TODO Sicherstellen, dass es java.lang.String ist.
    return Env().string_from_jstring( (jstring)as_jobject() );
}

//------------------------------------------------------------------------------replaced_characters

static string replaced_characters( const string& value, char oldChar, char newChar )  //TODO Verffentlichen
{
    string result = value;

    for( size_t i = 0; i < result.length(); i++ )
        if( result[i] == oldChar )
            result[i] = newChar;

    return result;
}

//--------------------------------------------------------------------------Value::signature_string

string Value::signature_string() const
{
    string result;

    if( _type == t_object ) {
        result = 'L' + java_class_path_of_name( Class::of_object( _jvalue.l )->name() ) + ';';
    } else {
        result += (char)_type;
    }

    return result;
}

//---------------------------------------------------------------------------java_class_path_of_name

string java_class_path_of_name( const string& className ) 
{
    return replaced_characters( className, '.', '/' );
}

//---------------------------------------------------------------------------------------Type::Type

Type::Type( const string& class_name )
: 
    _simple_type( t_object ), 
    _class_name(class_name) 
{
    check_class_name( _class_name );
}

//---------------------------------------------------------------------------------------Type::Type
    
Type::Type( const char* class_name )
: 
    _simple_type( t_object ), 
    _class_name(class_name) 
{
    check_class_name( _class_name );
}

//---------------------------------------------------------------------------Type::signature_string
    
string Type::signature_string() const 
{
    return _simple_type == t_object?
        'L' + java_class_path_of_name( _class_name ) + ';'
      : string( 1, (char)_simple_type );
}

//-------------------------------------------------------------------Type::of_type_signature_string
    
Type Type::of_type_signature_string( const string& signature )
{
    if( signature.length() == 1 ) {
        Signature::check_simple_type_char( signature[0] );
        return Type( (Simple_type)signature[0] );
    }
    else {
        Signature::check_class_signature_string( signature );
        return Type( signature.substr( 1, signature.length() - 2 ) );
    }
}

//---------------------------------------------------------------------------Type::check_class_name

void Type::check_class_name( const string& class_name )
{
    if( !is_class_name( class_name ) ) 
        throw_xc( "Z-JAVA-117", class_name );  
}

//------------------------------------------------------------------------------Type::is_class_name

bool Type::is_class_name( const string& class_name )
{
    if( class_name == "" )
        return false;

    //TODO  Hier vielleicht noch Klassensyntax prfen: Namen mit Punkten voneinander getrennt.

    return true;
}

//------------------------------------------------------------------------------Signature::of_types

Signature Signature::of_types( const Type& result_type ) 
{
    return Signature( "()" + result_type.signature_string() ); 
}

//------------------------------------------------------------------------------Signature::of_types

Signature Signature::of_types( const Type& result_type, const Type& t1 ) 
{
    return Signature( '(' + t1.signature_string() + ')' + result_type.signature_string() ); 
}

//-----------------------------------------------------------------------------Signature::Signature

Signature Signature::of_types( const Type& result_type, const Type& t1, const Type& t2 ) 
{
    return Signature( '(' + t1.signature_string() + t2.signature_string() + ')' + result_type.signature_string() ); 
}

//------------------------------------------------------------------------------Signature::of_types
    
Signature Signature::of_types( const Type& result_type, const Type& t1, const Type& t2, const Type& t3 ) 
{
    return Signature( '(' + t1.signature_string() + t2.signature_string() + t3.signature_string() + ')' + result_type.signature_string() );
}

//---------------------------------------------------------------------------Signature::return_type
    
Type Signature::return_type() const
{
    size_t end = _string.rfind( ')' );
    if( end == string::npos )
        throw_xc( Z_FUNCTION, "Ungltige Signatur" );

    return Type::of_type_signature_string( _string.substr( end + 1 ) );
}

//-----------------------------------------------------------------------Signature::parameter_count

int Signature::parameter_count() const
{
    assert( _string.length() >= 3 );
    assert( _string[0] == '(' );

    int result = 0;

    for( const char* p = _string.c_str() + 1; *p && *p != ')'; ) {
        if( *p == (char)t_object ) {       // zB "Ljava/lang/String;"
            p++;
            while( *p  &&  *p != ';' )  p++;
            if( *p )  p++;
            result++;
        }
        else {
            p++;
            result++;
        }
    }

    return result;
}

//----------------------------------------------------------------Signature::check_simple_type_char

void Signature::check_simple_type_char( char c )
{
    if( !is_simple_type_char( c ) ) 
        throw_xc( "Z-JAVA-116", c );
}

//-------------------------------------------------------------------Signature::is_simple_type_char

bool Signature::is_simple_type_char( char c )
{
    return 
        c == t_void    ||
        c == t_boolean ||
        c == t_byte    ||
        c == t_char    ||
        c == t_short   ||
        c == t_int     ||
        c == t_long    ||
        c == t_float   ||
        c == t_double;
}

//----------------------------------------------------------Signature::check_class_signature_string

void Signature::check_class_signature_string( const string& signature )
{
    if( !is_class_signature_string( signature ) )
        throw_xc( "Z-JAVA-115", signature );
}

//-------------------------------------------------------------Signature::is_class_signature_string

bool Signature::is_class_signature_string( const string& signature )
{
    if( signature.length() <= 3 )
        return false;

    if( signature[0] != 'L' )
        return false;

    if( *signature.rbegin() != ';' )
        return false;

    //TODO Hier vielleicht noch Klassensyntax prfen: Namen mit Schrgstrichen voneinander getrennt.

    return true;
}

//-------------------------------------------------------Variable_parameter_list::signature_string
    
string Variable_parameter_list::signature_string() const
{
    string result = "(";

    for( int i = 0; i < _size; i++ )
        result += _java_values[i].signature_string();
    result += ')';

    return result;
}

//------------------------------------------------------------------------------------Jvalue::check

void Jvalue::check( char type )
{
    if( _type != type )  
        throw_xc( "Z-JAVA-107", string( &type, 1 ), string( &_type, 1 ) );
}

//------------------------------------------------------------------------Jobject2::steal_local_ref

void Jobject2::steal_local_ref( jobject jo )
{
    if( jo != get_jobject() ) {
        assign( jo );

        if( jo ) {
            Env env;
            assert( env->GetObjectRefType( jo ) == JNILocalRefType );
            env->DeleteLocalRef( jo );
        }
    }
}

//------------------------------------------------------------------------------Jobject2::construct

void Jobject2::construct( const Signature& signature, const Parameter_list& parameter_list ) 
{
    assert( signature.return_simple_type() == t_void );
    value_call( "<init>", signature, parameter_list );
}

//------------------------------------------------------------------------------Jobject2::construct

void Jobject2::construct() 
{
    construct( Signature( "()V" ), Parameter_list::empty );
}

//-----------------------------------------------------------------------------Jobject2::value_call

Value Jobject2::value_call( const string& method_name, const Signature& signature, const Parameter_list& parameter_list ) const
{
    if( signature.parameter_count() != parameter_list.size() )
        throw_xc( Z_FUNCTION, method_name, "Parameteranzahl passt nicht" );

    //TODO Auer Anzahl auch die einzelnen Parametertypen vergleichen.

    Method method ( Class::of_object( get_jobject() ), method_name, signature );
    return method.value_call( get_jobject(), parameter_list );
}

//-------------------------------------------------------------------------------Method::value_call

Value Method::value_call( jobject jo, const Parameter_list& parameter_list ) const
{
    Value result;

    switch( signature().return_simple_type() ) {
        case t_void: 
            call( jo, parameter_list ); 
            break;

        case t_boolean:
            result.assign_jboolean( bool_call( jo, parameter_list ) );
            break;

        case t_byte:
            result.assign_jbyte( byte_call( jo, parameter_list ) );
            break;

        case t_char:
            result.assign_jchar( char_call( jo, parameter_list ) );
            break;

        case t_short:
            result.assign_jshort( short_call( jo, parameter_list ) );
            break;

        case t_int:
            result.assign_jint( int_call( jo, parameter_list ) );
            break;

        case t_long:
            result.assign_jlong( long_call( jo, parameter_list ) );
            break;

        case t_float:
            result.assign_jfloat( float_call( jo, parameter_list ) );
            break;

        case t_double:
            result.assign_jdouble( double_call( jo, parameter_list ) );
            break;

        case t_object:
            result.assign_jobject( object_call( jo, parameter_list ) );
            break;

        //case t_array:
        //    result.assign_jarray( object_call( jo, parameter_list ) );
        //    break;

        default:
            throw_xc( Z_FUNCTION );
    }

    return result;
}

//----------------------------------------------------------------------------------Procedure::call

void Procedure::call( jobject jo, const Parameter_list& parameter_list ) const
{
    Env env;

    env->CallVoidMethodA( jo, id(), parameter_list.jvalue_array() );
    if( env->ExceptionCheck() )  env.throw_java( "CallVoidMethodA" );
}

//--------------------------------------------------------------------------------Method::bool_call

bool Method::bool_call( jobject jo, const Parameter_list& parameter_list ) const
{
    Env env;

    jboolean result = env->CallBooleanMethodA( jo, _id, parameter_list.jvalue_array() );
    if( env->ExceptionCheck() )  env.throw_java( "CallBooleanMethodA" );

    return result != 0;
}

//--------------------------------------------------------------------------------Method::char_call

wchar_t Method::char_call( jobject jo, const Parameter_list& parameter_list ) const
{
    Env env;

    jchar result = env->CallCharMethodA( jo, _id, parameter_list.jvalue_array() );
    if( env->ExceptionCheck() )  env.throw_java( "CallCharMethodA" );

    return result;
}

//--------------------------------------------------------------------------------Method::byte_call

jbyte Method::byte_call( jobject jo, const Parameter_list& parameter_list ) const
{
    Env env;

    jbyte result = env->CallByteMethodA( jo, _id, parameter_list.jvalue_array() );
    if( env->ExceptionCheck() )  env.throw_java( "CallByteMethodA" );

    return result;
}

//-------------------------------------------------------------------------------Method::short_call

short Method::short_call( jobject jo, const Parameter_list& parameter_list ) const
{
    Env env;

    short result = env->CallShortMethodA( jo, _id, parameter_list.jvalue_array() );
    if( env->ExceptionCheck() )  env.throw_java( "CallShortMethodA" );

    return result;
}

//---------------------------------------------------------------------------------Method::int_call

int Method::int_call( jobject jo, const Parameter_list& parameter_list ) const
{
    Env env;

    int result = env->CallIntMethodA( jo, _id, parameter_list.jvalue_array() );
    if( env->ExceptionCheck() )  env.throw_java( "CallIntMethodA" );

    return result;
}

//--------------------------------------------------------------------------------Method::long_call

int64 Method::long_call( jobject jo, const Parameter_list& parameter_list ) const
{
    Env env;

    int64 result = env->CallLongMethodA( jo, _id, parameter_list.jvalue_array() );
    if( env->ExceptionCheck() )  env.throw_java( "CallLongMethodA" );

    return result;
}

//-------------------------------------------------------------------------------Method::float_call

float Method::float_call( jobject jo, const Parameter_list& parameter_list ) const
{
    Env env;

    float result = env->CallFloatMethodA( jo, _id, parameter_list.jvalue_array() );
    if( env->ExceptionCheck() )  env.throw_java( "CallFloatMethodA" );

    return result;
}

//------------------------------------------------------------------------------Method::double_call

double Method::double_call( jobject jo, const Parameter_list& parameter_list ) const
{
    Env env;

    double result = env->CallDoubleMethodA( jo, _id, parameter_list.jvalue_array() );
    if( env->ExceptionCheck() )  env.throw_java( "CallDoubleMethodA" );

    return result;
}

//------------------------------------------------------------------------------Method::object_call

Local_jobject Method::object_call( jobject jo, const Parameter_list& parameter_list ) const
{
    Env env;

    jobject result = env->CallObjectMethodA( jo, _id, parameter_list.jvalue_array() );
    if( env->ExceptionCheck() )  env.throw_java( "CallObjectMethodA" );

    return result;
}

//------------------------------------------------------------------------------Method::object_call

string Method::string_call( jobject jo, const Parameter_list& parameter_list ) const
{
    Env env;

    jstring jstr = (jstring)env->CallObjectMethodA( jo, _id, parameter_list.jvalue_array() );
    if( env->ExceptionCheck() )  env.throw_java( "CallObjectMethodA" );

    string result = env.string_from_jstring( jstr );

    env->DeleteLocalRef( jstr );
    return result;
}

//--------------------------------------------------------------------------Global_jobject2::assign

void Global_jobject2::assign( jobject jo )
{
    if( _jobject != jo ) 
    {
        Env env = jni_env();

        jobject new_jobject = NULL;

        if( jo )
        {
            Z_LOG2( Vm::java_log_category, "NewGlobalRef()\n" );

            new_jobject = env->NewGlobalRef( jo );
            if( !new_jobject || env->ExceptionCheck() )  env.throw_java( "NewGlobalRef" );
        }

        if( _jobject )
        {
            Z_LOG2( Vm::java_log_category, "DeleteGlobalRef()\n" );
            env->DeleteGlobalRef( _jobject );
            _jobject = NULL;
        }

        _jobject = new_jobject;
    }
}

//-------------------------------------------------------------------------Class_based::Class_based

Class_based::Class_based( Class* clas ) 
: 
    _class(clas)
{
    allocate();
}

//-------------------------------------------------------------------------Class_based::Class_based

Class_based::Class_based( jobject jo, Class* clas ) 
: 
    Global_jobject2( jo ),
    _class(clas)
{
    if( jo ) {
        Env env;
        jclass jo_class = env->GetObjectClass( jo );
        bool is_same = env->IsAssignableFrom( jo_class, clas->get_jclass() ) != 0;
        env->DeleteLocalRef( jo_class );
        if( !is_same )
            throw_xc( "Z-JAVA-118", clas->name() );
    }
}

//----------------------------------------------------------------------------Class_based::allocate
    
void Class_based::allocate()
{
    assert( !get_jobject() );
    steal_local_ref( _class->alloc_object() );
}

//------------------------------------------------------------------------------Class_factory::clas
    
Class* Class_factory::clas() const 
{ 
    return Vm::static_vm->_class_register.get_or_register_class( this ); 
}

//-------------------------------------------------------------------------------------------------

} //namespace javabridge
} //namespace zschimmer
